export declare class Slot<T = any> {
    type: SlotType<T>;
    /** @internal */ value: T;
    constructor(/** @internal */ type: SlotType<T>, 
    /** @internal */ value: T);
    static define<T>(): SlotType<T>;
    static get<T>(type: SlotType<T>, slots: readonly Slot[]): T | undefined;
}
export declare type SlotType<T> = (value: T) => Slot<T>;
declare const extensionBrand: unique symbol;
export declare type Extension = {
    [extensionBrand]: true;
} | ExtensionArray;
declare const enum Kind {
    Behavior = 0,
    Array = 1,
    Unique = 2,
    Name = 3
}
export declare type Behavior<Input, Output> = (value: Input) => Extension;
export declare class ExtensionGroup<Context> {
    readonly getStore: (context: Context) => {
        [id: number]: any;
    };
    private nextStorageID;
    constructor(getStore: (context: Context) => {
        [id: number]: any;
    });
    behavior<Input>(options?: {
        static?: boolean;
    }): Behavior<Input, readonly Input[]>;
    behavior<Input, Output>(options: {
        combine: (values: readonly Input[]) => Output;
        static?: boolean;
    }): Behavior<Input, Output>;
    dynamic<Input>(behavior: Behavior<Input, any>, read: (context: Context) => Input): Extension;
    unique<Spec>(instantiate: (specs: Spec[]) => Extension, defaultSpec?: Spec): (spec?: Spec) => Extension;
    resolve(extensions: readonly Extension[]): Configuration<Context>;
    resolveInner(extensions: readonly Extension[], replace?: readonly NamedExtensionValue[]): Configuration<Context>;
    defineName(): (extension: Extension) => Extension;
    storageID(): number;
    fallback: (extension: Extension) => Extension;
    normal: (extension: Extension) => Extension;
    extend: (extension: Extension) => Extension;
    override: (extension: Extension) => Extension;
}
interface ExtensionArray extends ReadonlyArray<Extension> {
}
declare class ExtensionValue {
    readonly kind: Kind;
    readonly id: any;
    readonly value: any;
    readonly type: ExtensionGroup<any> | null;
    readonly prec: number;
    constructor(kind: Kind, id: any, value: any, type: ExtensionGroup<any> | null, prec?: number);
    [extensionBrand]: true;
    collect(array: ExtensionValue[]): void;
}
declare type NamedExtensionValue = ExtensionValue & {
    kind: Kind.Name;
};
export declare class Configuration<Context> {
    private type;
    private extensions;
    private replaced;
    private readBehavior;
    readonly foreign: readonly Extension[];
    constructor(type: ExtensionGroup<Context>, extensions: readonly Extension[], replaced: readonly NamedExtensionValue[], readBehavior: {
        [id: number]: (context: Context) => any;
    }, foreign?: readonly Extension[]);
    getBehavior<Output>(behavior: Behavior<any, Output>, context?: Context): Output;
    replaceExtensions(replace: readonly Extension[]): Configuration<Context>;
}
export declare function combineConfig<Config>(configs: readonly Partial<Config>[], defaults: Partial<Config>, // Should hold only the optional properties of Config, but I haven't managed to express that
combine?: {
    [P in keyof Config]?: (first: Config[P], second: Config[P]) => Config[P];
}): Config;
export declare function fillConfig<Config>(config: Config, defaults: Partial<Config>): Required<Config>;
export {};
