import { Extension } from "../../extension";
import { EditorView, ViewUpdate, BlockInfo } from "../../view";
import { Range, RangeValue, RangeSet } from "../../rangeset";
import { ChangeSet, Annotation } from "../../state";
export declare abstract class GutterMarker extends RangeValue {
    compare(other: GutterMarker): boolean;
    abstract eq(other: GutterMarker): boolean;
    map(mapping: ChangeSet, pos: number): Range<GutterMarker> | null;
    toDOM(view: EditorView): Node | null;
    at(pos: number): Range<this>;
    elementClass: string;
}
declare type Handlers = {
    [event: string]: (view: EditorView, line: BlockInfo, event: any) => boolean;
};
export interface GutterConfig {
    style?: string;
    renderEmptyElements?: boolean;
    initialMarkers?: (view: EditorView) => RangeSet<GutterMarker>;
    updateMarkers?: (markers: RangeSet<GutterMarker>, update: ViewUpdate) => RangeSet<GutterMarker>;
    lineMarker?: (view: EditorView, line: BlockInfo, markers: readonly GutterMarker[]) => GutterMarker | null;
    initialSpacer?: null | ((view: EditorView) => GutterMarker);
    updateSpacer?: null | ((spacer: GutterMarker, update: ViewUpdate) => GutterMarker);
    handleDOMEvents?: Handlers;
}
export declare class Gutter {
    config: Required<GutterConfig>;
    constructor(config: GutterConfig);
    readonly extension: Extension[];
}
export declare const gutters: (spec?: {
    fixed?: boolean | undefined;
} | undefined) => Extension;
export interface LineNumberConfig {
    formatNumber?: (lineNo: number) => string;
    handleDOMEvents?: Handlers;
}
export declare const lineNumberMarkers: (value: LineNumberMarkerUpdate) => Annotation<LineNumberMarkerUpdate>;
export declare type LineNumberMarkerUpdate = {
    add?: Range<GutterMarker>[];
    filter?: (from: number, to: number, marker: GutterMarker) => boolean;
};
export declare const lineNumbers: (spec?: LineNumberConfig | undefined) => Extension;
export {};
