'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var state = require('../../state');
var view = require('../../view');
var styleMod = require('style-mod');

/// Returns an extension that enables multiple selections for the
/// editor. Secondary cursors and selected ranges are drawn with
/// simple decorations, and might look the same as the primary native
/// selection.
const multipleSelections = state.EditorState.extend.unique(() => {
    let rangeConfig = { class: styles.secondarySelection };
    return [
        state.EditorState.allowMultipleSelections(true),
        view.ViewPlugin.decoration({
            create(view) { return decorateSelections(view.state, rangeConfig); },
            update(deco, { prevState, state }) {
                return prevState.doc == state.doc && prevState.selection.eq(state.selection)
                    ? deco : decorateSelections(state, rangeConfig);
            }
        }),
        view.EditorView.styleModule(styles)
    ];
}, {});
class CursorWidget extends view.WidgetType {
    toDOM() {
        let span = document.createElement("span");
        span.className = styles.secondaryCursor;
        return span;
    }
}
function decorateSelections(state, rangeConfig) {
    let { ranges, primaryIndex } = state.selection;
    if (ranges.length == 1)
        return view.Decoration.none;
    let deco = [];
    for (let i = 0; i < ranges.length; i++)
        if (i != primaryIndex) {
            let range = ranges[i];
            deco.push(range.empty ? view.Decoration.widget(range.from, { widget: new CursorWidget(null) })
                : view.Decoration.mark(ranges[i].from, ranges[i].to, rangeConfig));
        }
    return view.Decoration.set(deco);
}
const styles = new styleMod.StyleModule({
    secondarySelection: {
        backgroundColor_fallback: "#3297FD",
        color_fallback: "white !important",
        backgroundColor: "Highlight",
        color: "HighlightText !important"
    },
    secondaryCursor: {
        display: "inline-block",
        verticalAlign: "text-top",
        borderLeft: "1px solid #555",
        width: 0,
        height: "1.15em",
        margin: "0 -0.5px -.5em"
    }
});

exports.multipleSelections = multipleSelections;
