import { ChangeSet, ChangedRange } from "../../state";
export declare abstract class RangeValue {
    abstract map(mapping: ChangeSet, from: number, to: number): Range<any> | null;
    eq(other: RangeValue): boolean;
    startSide: number;
    endSide: number;
    point: boolean;
}
export interface RangeComparator<T extends RangeValue> {
    compareRange(from: number, to: number, activeA: T[], activeB: T[]): void;
    comparePoint(from: number, to: number, byA: T, byB: T | null): void;
}
export interface RangeIterator<T extends RangeValue> {
    span(from: number, to: number, active: readonly T[]): void;
    point(from: number, to: number, value: T, openStart: boolean, openEnd: boolean): void;
    ignore(from: number, to: number, value: T): boolean;
}
export declare class Range<T extends RangeValue> {
    readonly from: number;
    readonly to: number;
    readonly value: T;
    constructor(from: number, to: number, value: T);
    map(changes: ChangeSet, oldOffset: number, newOffset: number): Range<T> | null;
    move(offset: number): Range<T>;
    readonly heapPos: number;
    readonly heapSide: number;
}
declare type RangeFilter<T> = (from: number, to: number, value: T) => boolean;
export declare class RangeSet<T extends RangeValue> {
    length: number;
    size: number;
    local: readonly Range<T>[];
    children: readonly RangeSet<T>[];
    constructor(length: number, size: number, local: readonly Range<T>[], children: readonly RangeSet<T>[]);
    update(added?: readonly Range<T>[], filter?: RangeFilter<T> | null, filterFrom?: number, filterTo?: number): RangeSet<T>;
    updateInner(added: readonly Range<T>[], filter: RangeFilter<T> | null, filterFrom: number, filterTo: number, offset: number, length: number): RangeSet<T>;
    grow(length: number): RangeSet<T>;
    collect(target: Range<T>[], offset: number): void;
    map(changes: ChangeSet): RangeSet<T>;
    private mapInner;
    between(from: number, to: number, f: (from: number, to: number, value: T) => void): void;
    betweenInner(from: number, to: number, f: (from: number, to: number, value: T) => void, offset: number): void;
    iter(from?: number, to?: number): {
        next: () => Range<T> | void;
    };
    compare(other: RangeSet<T>, textDiff: readonly ChangedRange[], comparator: RangeComparator<T>, oldLen: number): void;
    static iterateSpans<T extends RangeValue>(sets: readonly RangeSet<T>[], from: number, to: number, iterator: RangeIterator<T>): void;
    static of<T extends RangeValue>(ranges: readonly Range<T>[] | Range<T>): RangeSet<T>;
    static empty: RangeSet<any>;
}
export {};
