import { EditorState } from "./state";
import { Transaction } from "./transaction";
import { Extension, ExtensionGroup } from "../../extension";
import { Tree, NodeType } from "lezer-tree";
export declare type StateCommand = (target: {
    state: EditorState;
    dispatch: (transaction: Transaction) => void;
}) => boolean;
export declare const extendState: ExtensionGroup<EditorState>;
export declare const stateField: import("../../extension/dist/extension").Behavior<StateField<any>, readonly StateField<any>[]>;
export declare const allowMultipleSelections: import("../../extension/dist/extension").Behavior<boolean, boolean>;
export interface StateFieldSpec<Value> {
    init: (state: EditorState) => Value;
    apply: (tr: Transaction, value: Value, newState: EditorState) => Value;
}
export declare class StateField<Value> {
    readonly extension: Extension;
    readonly id: number;
    readonly init: (state: EditorState) => Value;
    readonly apply: (tr: Transaction, value: Value, state: EditorState) => Value;
    constructor(spec: StateFieldSpec<Value>);
}
export declare class Annotation<T> {
    type: (value: T) => Annotation<T>;
    /** @internal */ value: T;
    constructor(/** @internal */ type: (value: T) => Annotation<T>, 
    /** @internal */ value: T);
    static define<T>(): (value: T) => Annotation<T>;
}
export declare type CancellablePromise<T> = Promise<T> & {
    canceled?: boolean;
};
export interface Syntax {
    getTree(state: EditorState, from: number, to: number): {
        tree: Tree;
        rest: CancellablePromise<Tree> | null;
    };
    tryGetTree(state: EditorState, from: number, to: number): Tree | null;
    getPartialTree(state: EditorState, from: number, to: number): Tree;
    docTypeAt(state: EditorState, pos: number): NodeType;
}
