import { Text } from "../../text";
import { EditorSelection } from "./selection";
import { Transaction } from "./transaction";
import { Extension, Configuration, Behavior } from "../../extension";
import { Syntax, StateField } from "./extension";
export interface EditorStateConfig {
    doc?: string | Text;
    selection?: EditorSelection;
    extensions?: readonly Extension[];
}
export declare class EditorState {
    readonly configuration: Configuration<EditorState>;
    readonly values: {
        [id: number]: any;
    };
    readonly doc: Text;
    readonly selection: EditorSelection;
    constructor(configuration: Configuration<EditorState>, values: {
        [id: number]: any;
    }, doc: Text, selection: EditorSelection);
    field<T>(field: StateField<T>): T;
    field<T>(field: StateField<T>, require: false): T | undefined;
    applyTransaction(tr: Transaction): EditorState;
    t(timestamp?: number): Transaction;
    joinLines(text: readonly string[]): string;
    splitLines(text: string): string[];
    behavior<Output>(behavior: Behavior<any, Output>): Output;
    toJSON(): any;
    static fromJSON(json: any, config?: EditorStateConfig): EditorState;
    static create(config?: EditorStateConfig): EditorState;
    static extend: import("../../extension/dist/extension").ExtensionGroup<EditorState>;
    static allowMultipleSelections: Behavior<boolean, boolean>;
    static indentation: Behavior<(state: EditorState, pos: number) => number, readonly ((state: EditorState, pos: number) => number)[]>;
    static tabSize: Behavior<number, number>;
    readonly tabSize: number;
    static lineSeparator: Behavior<string, string | undefined>;
    static indentUnit: Behavior<number, number>;
    readonly indentUnit: number;
    static syntax: Behavior<Syntax, readonly Syntax[]>;
    static foldable: Behavior<(state: EditorState, lineStart: number, lineEnd: number) => {
        from: number;
        to: number;
    } | null, readonly ((state: EditorState, lineStart: number, lineEnd: number) => {
        from: number;
        to: number;
    } | null)[]>;
}
