import { ChangeSet, ChangedRange } from "../../state";
import { RangeValue, Range, RangeSet } from "../../rangeset";
import { WidgetView } from "./inlineview";
import { EditorView } from "./editorview";
export interface MarkDecorationSpec {
    inclusive?: boolean;
    inclusiveStart?: boolean;
    inclusiveEnd?: boolean;
    attributes?: {
        [key: string]: string;
    };
    class?: string;
    tagName?: string;
}
export interface WidgetDecorationSpec {
    widget: WidgetType;
    side?: number;
    block?: boolean;
}
export interface ReplaceDecorationSpec {
    widget?: WidgetType;
    inclusive?: boolean;
    inclusiveStart?: boolean;
    inclusiveEnd?: boolean;
    block?: boolean;
}
export interface LineDecorationSpec {
    attributes?: {
        [key: string]: string;
    };
}
export declare abstract class WidgetType<T = any> {
    readonly value: T;
    constructor(value: T);
    abstract toDOM(view: EditorView): HTMLElement;
    eq(value: T): boolean;
    updateDOM(dom: HTMLElement): boolean;
    compare(other: WidgetType): boolean;
    readonly estimatedHeight: number;
    ignoreEvent(event: Event): boolean;
    readonly customView: null | typeof WidgetView;
}
export declare type DecorationSet = RangeSet<Decoration>;
export declare enum BlockType {
    Text = 0,
    WidgetBefore = 1,
    WidgetAfter = 2,
    WidgetRange = 3
}
export declare abstract class Decoration extends RangeValue {
    readonly startSide: number;
    readonly endSide: number;
    readonly widget: WidgetType | null;
    readonly spec: any;
    constructor(startSide: number, endSide: number, widget: WidgetType | null, spec: any);
    readonly point: boolean;
    readonly heightRelevant: boolean;
    abstract map(mapping: ChangeSet, from: number, to: number): Range<Decoration> | null;
    abstract eq(other: Decoration): boolean;
    static mark(from: number, to: number, spec: MarkDecorationSpec): Range<Decoration>;
    static widget(pos: number, spec: WidgetDecorationSpec): Range<Decoration>;
    static replace(from: number, to: number, spec: ReplaceDecorationSpec): Range<Decoration>;
    static line(start: number, spec: LineDecorationSpec): Range<Decoration>;
    static set(of: Range<Decoration> | readonly Range<Decoration>[]): DecorationSet;
    static none: RangeSet<Decoration>;
    hasHeight(): boolean;
    mapSimple(mapping: ChangeSet, from: number, to: number): Range<this> | null;
}
export declare class MarkDecoration extends Decoration {
    constructor(spec: MarkDecorationSpec);
    map(mapping: ChangeSet, from: number, to: number): Range<Decoration> | null;
    eq(other: Decoration): boolean;
}
export declare class LineDecoration extends Decoration {
    constructor(spec: LineDecorationSpec);
    readonly point: boolean;
    map(mapping: ChangeSet, pos: number): Range<Decoration> | null;
    eq(other: Decoration): boolean;
}
export declare class PointDecoration extends Decoration {
    block: boolean;
    constructor(spec: any, startSide: number, endSide: number, block: boolean, widget: WidgetType | null);
    readonly point: boolean;
    readonly type: BlockType.WidgetBefore | BlockType.WidgetAfter | BlockType.WidgetRange;
    readonly heightRelevant: boolean;
    map(mapping: ChangeSet, from: number, to: number): Range<Decoration> | null;
    eq(other: Decoration): boolean;
}
export declare function joinRanges(a: number[], b: number[]): number[];
declare class Changes {
    content: number[];
    height: number[];
}
export declare function findChangedRanges(a: DecorationSet, b: DecorationSet, diff: readonly ChangedRange[], lengthA: number): Changes;
export declare function heightRelevantDecorations(decorations: readonly DecorationSet[], ranges: readonly ChangedRange[]): number[];
export {};
