import { EditorState, Transaction, CancellablePromise, Annotation } from "../../state";
import { Configuration, Extension, Behavior } from "../../extension";
import { StyleModule, Style } from "style-mod";
import { DocView } from "./docview";
import { InputState, MouseSelectionUpdate } from "./input";
import { Rect } from "./dom";
import { BlockInfo } from "./heightmap";
import { Viewport } from "./viewport";
import { ViewUpdate, ViewPlugin, ViewPluginValue } from "./extension";
import { Attrs } from "./attributes";
export interface EditorConfig {
    state?: EditorState;
    extensions?: Extension[];
    root?: Document | ShadowRoot;
    dispatch?: (tr: Transaction) => void;
}
export declare const enum UpdateState {
    Idle = 0,
    Measuring = 1,
    Updating = 2
}
export declare class EditorView {
    private _state;
    readonly state: EditorState;
    _viewport: Viewport;
    readonly viewport: {
        from: number;
        to: number;
    };
    readonly dispatch: (tr: Transaction) => void;
    readonly root: DocumentOrShadowRoot;
    readonly dom: HTMLElement;
    readonly scrollDOM: HTMLElement;
    readonly contentDOM: HTMLElement;
    inputState: InputState;
    readonly docView: DocView;
    private extensions;
    configuration: Configuration<EditorView>;
    plugins: {
        [id: number]: any;
    };
    private editorAttrs;
    private contentAttrs;
    private styleModules;
    private themeCache;
    private themeCacheFor;
    updateState: UpdateState;
    waiting: CancellablePromise<any>[];
    static extend: import("../../extension/dist/extension").ExtensionGroup<EditorView>;
    constructor(config?: EditorConfig);
    update(transactions?: Transaction[], annotations?: Annotation<any>[]): void;
    waitFor(promise: CancellablePromise<any>): void;
    private clearWaiting;
    updateAttrs(): void;
    private configure;
    private updatePlugins;
    private mountStyles;
    drawPlugins(): void;
    plugin<T extends ViewPluginValue>(plugin: ViewPlugin<T>): T | undefined;
    behavior<Output>(behavior: Behavior<any, Output>): Output;
    updateInner(update: ViewUpdate, viewport: Viewport): void;
    cssClass(selector: string): string;
    phrase(phrase: string): string;
    domAtPos(pos: number): {
        node: Node;
        offset: number;
    };
    posAtDOM(node: Node, offset?: number): number;
    private readingLayout;
    blockAtHeight(height: number, editorTop?: number): BlockInfo;
    lineAtHeight(height: number, editorTop?: number): BlockInfo;
    lineAt(pos: number, editorTop?: number): BlockInfo;
    viewportLines(f: (height: BlockInfo) => void, editorTop?: number): void;
    readonly contentHeight: number;
    movePos(start: number, direction: "forward" | "backward" | "left" | "right", granularity?: "character" | "word" | "line" | "lineboundary", action?: "move" | "extend"): number;
    posAtCoords(coords: {
        x: number;
        y: number;
    }): number;
    coordsAtPos(pos: number): Rect | null;
    readonly defaultCharacterWidth: number;
    readonly defaultLineHeight: number;
    startMouseSelection(event: MouseEvent, update: MouseSelectionUpdate): void;
    readonly hasFocus: boolean;
    focus(): void;
    destroy(): void;
    static styleModule: Behavior<StyleModule<{}>, readonly StyleModule<{}>[]>;
    static handleDOMEvents: Behavior<{
        [key: string]: (view: EditorView, event: any) => boolean;
    }, readonly {
        [key: string]: (view: EditorView, event: any) => boolean;
    }[]>;
    static dragMovesSelection: Behavior<(event: MouseEvent) => boolean, readonly ((event: MouseEvent) => boolean)[]>;
    static clickAddsSelectionRange: Behavior<(event: MouseEvent) => boolean, readonly ((event: MouseEvent) => boolean)[]>;
    static decorations: Behavior<import("../../rangeset/dist/rangeset").RangeSet<import("./decoration").Decoration>, readonly import("../../rangeset/dist/rangeset").RangeSet<import("./decoration").Decoration>[]>;
    static theme(spec: {
        [name: string]: Style;
    }): Extension;
    static phrases: Behavior<{
        [key: string]: string;
    }, readonly {
        [key: string]: string;
    }[]>;
    static scrollMargins: Behavior<{
        left?: number | undefined;
        top?: number | undefined;
        right?: number | undefined;
        bottom?: number | undefined;
    }, Rect>;
    static contentAttributes: Behavior<Attrs, Attrs>;
    static editorAttributes: Behavior<Attrs, Attrs>;
    static focusChange: (value: boolean) => Annotation<boolean>;
}
