import { EditorState, Transaction, ChangeSet, Annotation } from "../../state";
import { StyleModule } from "style-mod";
import { Viewport } from "./viewport";
import { DecorationSet } from "./decoration";
import { Extension, Behavior, ExtensionGroup } from "../../extension";
import { EditorView } from "./editorview";
import { Attrs } from "./attributes";
import { Rect } from "./dom";
export declare type Command = (target: EditorView) => boolean;
export declare const extendView: ExtensionGroup<EditorView>;
export declare const handleDOMEvents: Behavior<{
    [key: string]: (view: EditorView, event: any) => boolean;
}, readonly {
    [key: string]: (view: EditorView, event: any) => boolean;
}[]>;
export declare const clickAddsSelectionRange: Behavior<(event: MouseEvent) => boolean, readonly ((event: MouseEvent) => boolean)[]>;
export declare const dragMovesSelection: Behavior<(event: MouseEvent) => boolean, readonly ((event: MouseEvent) => boolean)[]>;
export declare class ViewPlugin<Value extends ViewPluginValue<any>> {
    readonly create: (view: EditorView) => Value;
    readonly id: number;
    readonly behaviorExtensions: readonly Extension[];
    private constructor();
    readonly extension: Extension;
    static create<Value extends ViewPluginValue<any>>(create: (view: EditorView) => Value): ViewPlugin<Value>;
    behavior<Input>(behavior: Behavior<Input, any>, read: (plugin: Value) => Input): ViewPlugin<Value>;
    decorations(read: (plugin: Value) => DecorationSet): ViewPlugin<Value>;
    static decoration(spec: DecorationPluginSpec): Extension;
}
export interface DecorationPluginSpec {
    create: (view: EditorView) => DecorationSet;
    update: (deco: DecorationSet, update: ViewUpdate) => DecorationSet;
    map?: boolean;
}
export interface ViewPluginValue<Measure = undefined> {
    update?(update: ViewUpdate): void;
    draw?(): void;
    measure?(): Measure;
    drawMeasured?(measured: Measure): void;
    destroy?(): void;
}
export declare const editorAttributes: Behavior<Attrs, Attrs>;
export declare const contentAttributes: Behavior<Attrs, Attrs>;
export declare const viewPlugin: Behavior<ViewPlugin<any>, readonly ViewPlugin<any>[]>;
export declare const decorations: Behavior<import("../../rangeset/dist/rangeset").RangeSet<import("./decoration").Decoration>, readonly import("../../rangeset/dist/rangeset").RangeSet<import("./decoration").Decoration>[]>;
export declare const styleModule: Behavior<StyleModule<{}>, readonly StyleModule<{}>[]>;
export declare const theme: Behavior<StyleModule<{
    [key: string]: string;
}>, readonly StyleModule<{
    [key: string]: string;
}>[]>;
export declare const phrases: Behavior<{
    [key: string]: string;
}, readonly {
    [key: string]: string;
}[]>;
export declare const scrollMargins: Behavior<{
    left?: number | undefined;
    top?: number | undefined;
    right?: number | undefined;
    bottom?: number | undefined;
}, Rect>;
export declare const focusChange: (value: boolean) => Annotation<boolean>;
export declare const notified: (value: boolean) => Annotation<boolean>;
export declare class ViewUpdate {
    readonly view: EditorView;
    readonly transactions: readonly Transaction[];
    readonly _annotations: readonly Annotation<any>[];
    readonly state: EditorState;
    readonly changes: ChangeSet;
    readonly prevState: EditorState;
    readonly prevViewport: Viewport;
    private prevThemes;
    constructor(view: EditorView, transactions?: readonly Transaction[], _annotations?: readonly Annotation<any>[]);
    readonly viewport: {
        from: number;
        to: number;
    };
    readonly viewportChanged: boolean;
    readonly docChanged: boolean;
    readonly themeChanged: boolean;
    annotation<T>(type: (value: T) => Annotation<T>): T | undefined;
    annotations<T>(type: (value: T) => Annotation<T>): readonly T[];
}
