import { Text } from "../../text";
import { ChangedRange } from "../../state";
import { DecorationSet, BlockType } from "./decoration";
export declare class HeightOracle {
    doc: Text;
    lineWrapping: boolean;
    heightSamples: {
        [key: number]: boolean;
    };
    lineHeight: number;
    charWidth: number;
    lineLength: number;
    heightChanged: boolean;
    heightForGap(from: number, to: number): number;
    heightForLine(length: number): number;
    setDoc(doc: Text): this;
    mustRefresh(lineHeights: number[]): boolean;
    refresh(whiteSpace: string, lineHeight: number, charWidth: number, lineLength: number, knownHeights: number[]): boolean;
}
export declare class MeasuredHeights {
    readonly from: number;
    readonly heights: number[];
    index: number;
    constructor(from: number, heights: number[]);
    readonly more: boolean;
}
export declare class BlockInfo {
    readonly from: number;
    readonly length: number;
    readonly top: number;
    readonly height: number;
    readonly type: BlockType | readonly BlockInfo[];
    constructor(from: number, length: number, top: number, height: number, type: BlockType | readonly BlockInfo[]);
    readonly to: number;
    readonly bottom: number;
    join(other: BlockInfo): BlockInfo;
}
export declare enum QueryType {
    ByPos = 0,
    ByHeight = 1,
    ByPosNoHeight = 2
}
export declare abstract class HeightMap {
    length: number;
    height: number;
    flags: number;
    constructor(length: number, // The number of characters covered
    height: number, // Height of this part of the document
    flags?: number);
    size: number;
    outdated: boolean;
    abstract blockAt(height: number, doc: Text, top: number, offset: number): BlockInfo;
    abstract lineAt(value: number, type: QueryType, doc: Text, top: number, offset: number): BlockInfo;
    abstract forEachLine(from: number, to: number, doc: Text, top: number, offset: number, f: (line: BlockInfo) => void): void;
    abstract updateHeight(oracle: HeightOracle, offset?: number, force?: boolean, measured?: MeasuredHeights): HeightMap;
    abstract toString(): void;
    setHeight(oracle: HeightOracle, height: number): void;
    replace(from: number, to: number, nodes: (HeightMap | null)[]): HeightMap;
    decomposeLeft(to: number, result: (HeightMap | null)[]): void;
    decomposeRight(from: number, result: (HeightMap | null)[]): void;
    applyChanges(decorations: readonly DecorationSet[], oldDoc: Text, oracle: HeightOracle, changes: readonly ChangedRange[]): HeightMap;
    static empty(): HeightMapText;
    static of(nodes: (HeightMap | null)[]): HeightMap;
}
declare class HeightMapBlock extends HeightMap {
    readonly type: BlockType;
    constructor(length: number, height: number, type: BlockType);
    blockAt(height: number, doc: Text, top: number, offset: number): BlockInfo;
    lineAt(value: number, type: QueryType, doc: Text, top: number, offset: number): BlockInfo;
    forEachLine(from: number, to: number, doc: Text, top: number, offset: number, f: (line: BlockInfo) => void): void;
    updateHeight(oracle: HeightOracle, offset?: number, force?: boolean, measured?: MeasuredHeights): this;
    toString(): string;
}
declare class HeightMapText extends HeightMapBlock {
    collapsed: number;
    widgetHeight: number;
    constructor(length: number, height: number);
    replace(from: number, to: number, nodes: (HeightMap | null)[]): HeightMap;
    updateHeight(oracle: HeightOracle, offset?: number, force?: boolean, measured?: MeasuredHeights): this;
    toString(): string;
}
export {};
